@@ -103,270 +103,272 @@ write_objects <- function(clustassess_object,
         metadata_file_name
     )
 
-    # establish the feature ordering (original and stable) and convert to data.table
-    feature_ordering <- list(original = list(), stable = list(), original_incremental = list())
-    resolution_values <- c()
+    if (!is.null(clustassess_object)) {
+        # establish the feature ordering (original and stable) and convert to data.table
+        feature_ordering <- list(original = list(), stable = list(), original_incremental = list())
+        resolution_values <- c()
 
-    ftype_index <- 1
-    nftypes <- length(clustassess_object$feature_stability$by_steps)
-    unique_n_colors <- c(nftypes)
+        ftype_index <- 1
+        nftypes <- length(clustassess_object$feature_stability$by_steps)
+        unique_n_colors <- c(nftypes)
 
-    clustassess_object$feature_stability$colours <- generate_colours(nftypes, qualpalr_colorspace)
+        clustassess_object$feature_stability$colours <- generate_colours(nftypes, qualpalr_colorspace)
 
-    for (ftype in names(clustassess_object$feature_stability$by_steps)) {
-        feature_ordering$original[[ftype]] <- names(clustassess_object$feature_stability$by_steps[[ftype]])
-        fsize_index <- 1
-        upper_limit <- length(clustassess_object$feature_stability$incremental[[ftype]])
+        for (ftype in names(clustassess_object$feature_stability$by_steps)) {
+            feature_ordering$original[[ftype]] <- names(clustassess_object$feature_stability$by_steps[[ftype]])
+            fsize_index <- 1
+            upper_limit <- length(clustassess_object$feature_stability$incremental[[ftype]])
 
-        for (fsize in names(clustassess_object$feature_stability$by_steps[[ftype]])) {
-            resolution_values <- union(resolution_values, names(clustassess_object$feature_stability$by_steps[[ftype]][[fsize]]))
-
-            summary_values_by_step <- rep(0, length(clustassess_object$feature_stability$by_steps[[ftype]][[fsize_index]]))
-            if (fsize_index <= upper_limit) {
-                summary_values_incremental <- rep(0, length(clustassess_object$feature_stability$incremental[[ftype]][[fsize_index]]))
-            }
-
-            res_index <- 1
-
-            for (resval in names(clustassess_object$feature_stability$by_steps[[ftype]][[fsize]])) {
-                ecc_by_step <- clustassess_object$feature_stability$by_steps[[ftype]][[fsize_index]][[resval]]$ecc
-                summary_values_by_step[res_index] <- summary_function(ecc_by_step)
-                res_dt_by_step <- data.frame(ecc = ecc_by_step, res = resval) #  mb = mb if you want to also include the mb vector
-                if (res_index == 1) {
-                    size_dt_by_step <- res_dt_by_step
-                } else {
-                    size_dt_by_step <- rbind(size_dt_by_step, res_dt_by_step)
-                }
+            for (fsize in names(clustassess_object$feature_stability$by_steps[[ftype]])) {
+                resolution_values <- union(resolution_values, names(clustassess_object$feature_stability$by_steps[[ftype]][[fsize]]))
 
+                summary_values_by_step <- rep(0, length(clustassess_object$feature_stability$by_steps[[ftype]][[fsize_index]]))
                 if (fsize_index <= upper_limit) {
-                    ecc_incremental <- clustassess_object$feature_stability$incremental[[ftype]][[fsize_index]][[resval]]
-
-                    summary_values_incremental[res_index] <- summary_function(ecc_incremental)
-                    res_dt_incremental <- data.frame(ecc = ecc_incremental, res = resval) #  mb = mb if you want to also include the mb vector
-                    if (res_index == 1) {
-                        size_dt_incremental <- res_dt_incremental
-                    } else {
-                        size_dt_incremental <- rbind(size_dt_incremental, res_dt_incremental)
-                    }
+                    summary_values_incremental <- rep(0, length(clustassess_object$feature_stability$incremental[[ftype]][[fsize_index]]))
                 }
 
-                res_index <- res_index + 1
+                res_index <- 1
+
+                for (resval in names(clustassess_object$feature_stability$by_steps[[ftype]][[fsize]])) {
+                    ecc_by_step <- clustassess_object$feature_stability$by_steps[[ftype]][[fsize_index]][[resval]]$ecc
+                    summary_values_by_step[res_index] <- summary_function(ecc_by_step)
+                    res_dt_by_step <- data.frame(ecc = ecc_by_step, res = resval) #  mb = mb if you want to also include the mb vector
+                    if (res_index == 1) {
+                        size_dt_by_step <- res_dt_by_step
+                    } else {
+                        size_dt_by_step <- rbind(size_dt_by_step, res_dt_by_step)
+                    }
+
+                    if (fsize_index <= upper_limit) {
+                        ecc_incremental <- clustassess_object$feature_stability$incremental[[ftype]][[fsize_index]][[resval]]
+
+                        summary_values_incremental[res_index] <- summary_function(ecc_incremental)
+                        res_dt_incremental <- data.frame(ecc = ecc_incremental, res = resval) #  mb = mb if you want to also include the mb vector
+                        if (res_index == 1) {
+                            size_dt_incremental <- res_dt_incremental
+                        } else {
+                            size_dt_incremental <- rbind(size_dt_incremental, res_dt_incremental)
+                        }
+                    }
+
+                    res_index <- res_index + 1
+                }
+
+                size_dt_by_step[, "fsize"] <- fsize_index
+                summary_values_by_step <- data.frame(ecc = summary_values_by_step, fsize = fsize_index)
+
+                if (fsize_index == 1) {
+                    ftype_dt_by_step <- size_dt_by_step
+                    ftype_summary_dt_by_step <- summary_values_by_step
+                } else {
+                    ftype_dt_by_step <- rbind(ftype_dt_by_step, size_dt_by_step)
+                    ftype_summary_dt_by_step <- rbind(ftype_summary_dt_by_step, summary_values_by_step)
+                }
+
+                if (fsize_index > upper_limit) {
+                    next
+                }
+
+                size_dt_incremental[, "fsize"] <- fsize_index
+                summary_values_incremental <- data.frame(ecc = summary_values_incremental, fsize = fsize_index)
+
+                if (fsize_index == 1) {
+                    ftype_dt_incremental <- size_dt_incremental
+                    ftype_summary_dt_incremental <- summary_values_incremental
+                } else {
+                    ftype_dt_incremental <- rbind(ftype_dt_incremental, size_dt_incremental)
+                    ftype_summary_dt_incremental <- rbind(ftype_summary_dt_incremental, summary_values_incremental)
+                }
+
+                fsize_index <- fsize_index + 1
             }
 
-            size_dt_by_step[, "fsize"] <- fsize_index
-            summary_values_by_step <- data.frame(ecc = summary_values_by_step, fsize = fsize_index)
+            ftype_dt_by_step[, "ftype"] <- ftype
+            ftype_dt_incremental[, "ftype"] <- ftype
 
-            if (fsize_index == 1) {
-                ftype_dt_by_step <- size_dt_by_step
-                ftype_summary_dt_by_step <- summary_values_by_step
+            ftype_summary_dt_by_step[, "ftype"] <- ftype
+            ftype_summary_dt_incremental[, "ftype"] <- ftype
+            if (ftype_index == 1) {
+                overall_dtable_by_step <- ftype_dt_by_step
+                overall_dtable_incremental <- ftype_dt_incremental
+                overall_summary_dt_by_step <- ftype_summary_dt_by_step
+                overall_summary_dt_incremental <- ftype_summary_dt_incremental
             } else {
-                ftype_dt_by_step <- rbind(ftype_dt_by_step, size_dt_by_step)
-                ftype_summary_dt_by_step <- rbind(ftype_summary_dt_by_step, summary_values_by_step)
+                overall_dtable_by_step <- rbind(overall_dtable_by_step, ftype_dt_by_step)
+                overall_dtable_incremental <- rbind(overall_dtable_incremental, ftype_dt_incremental)
+                overall_summary_dt_by_step <- rbind(overall_summary_dt_by_step, ftype_summary_dt_by_step)
+                overall_summary_dt_incremental <- rbind(overall_summary_dt_incremental, ftype_summary_dt_incremental)
             }
 
-            if (fsize_index > upper_limit) {
-                next
+            ftype_index <- ftype_index + 1
+        }
+
+        feature_ordering$resolution <- stringr::str_sort(resolution_values, numeric = TRUE)
+        # split the data tables by resolution
+        clustassess_object$feature_stability$by_steps <- lapply(feature_ordering$resolution, function(resval) {
+            subdt <- overall_dtable_by_step %>% dplyr::filter(.data$res == resval) # %>% dplyr::arrange(order(.data$ecc))
+            subdt$fsize <- factor(subdt$fsize)
+            subdt$resval <- NULL
+
+            return(subdt)
+        })
+        names(clustassess_object$feature_stability$by_steps) <- feature_ordering$resolution
+
+        clustassess_object$feature_stability$incremental <- lapply(feature_ordering$resolution, function(resval) {
+            subdt <- overall_dtable_incremental %>%
+                dplyr::filter(.data$res == resval) %>%
+                dplyr::arrange(order(.data$ecc))
+            subdt$fsize <- factor(subdt$fsize)
+
+            return(subdt)
+        })
+        names(clustassess_object$feature_stability$incremental) <- feature_ordering$resolution
+
+        clustassess_object$feature_stability$overall <- list(
+            by_step = overall_summary_dt_by_step,
+            incremental = overall_summary_dt_incremental
+        )
+
+        # store the names and ordering of the stable feature sizes
+        for (ftype in names(feature_ordering$original)) {
+            nsteps <- length(clustassess_object[[ftype]]) - 1
+            feature_ordering$stable[[ftype]] <- names(clustassess_object[[ftype]])[seq_len(nsteps)]
+
+            feature_ordering$original_incremental[[ftype]] <- rep(0, length(feature_ordering$original[[ftype]]) - 1)
+
+            for (fsize_index in seq_len(length(feature_ordering$original[[ftype]]) - 1)) {
+                feature_ordering$original_incremental[[ftype]][fsize_index] <- paste(
+                    feature_ordering$original[[ftype]][fsize_index],
+                    feature_ordering$original[[ftype]][fsize_index + 1],
+                    sep = "-"
+                )
             }
-
-            size_dt_incremental[, "fsize"] <- fsize_index
-            summary_values_incremental <- data.frame(ecc = summary_values_incremental, fsize = fsize_index)
-
-            if (fsize_index == 1) {
-                ftype_dt_incremental <- size_dt_incremental
-                ftype_summary_dt_incremental <- summary_values_incremental
-            } else {
-                ftype_dt_incremental <- rbind(ftype_dt_incremental, size_dt_incremental)
-                ftype_summary_dt_incremental <- rbind(ftype_summary_dt_incremental, summary_values_incremental)
-            }
-
-            fsize_index <- fsize_index + 1
         }
 
-        ftype_dt_by_step[, "ftype"] <- ftype
-        ftype_dt_incremental[, "ftype"] <- ftype
+        # ---
+        # create the clustering data tables
+        # ecc_by_k_dt - columns: clustering method, k and ecc
+        # ecc_by_res_dt - columns: clustering method, res and ecc
+        for (ftype in names(feature_ordering$stable)) {
+            for (fsize in feature_ordering$stable[[ftype]]) {
+                # remove the adjacency matrix, not needed for the shiny app
+                clustassess_object[[ftype]][[fsize]]$adj_matrix <- NULL
+                mbs_list <- list()
+                ecc_list <- list()
+                ecc_list_by_res <- list()
+                ecc_order_list <- list()
+                ecc_order_list_by_res <- list()
+                structure_list <- list()
 
-        ftype_summary_dt_by_step[, "ftype"] <- ftype
-        ftype_summary_dt_incremental[, "ftype"] <- ftype
-        if (ftype_index == 1) {
-            overall_dtable_by_step <- ftype_dt_by_step
-            overall_dtable_incremental <- ftype_dt_incremental
-            overall_summary_dt_by_step <- ftype_summary_dt_by_step
-            overall_summary_dt_incremental <- ftype_summary_dt_incremental
-        } else {
-            overall_dtable_by_step <- rbind(overall_dtable_by_step, ftype_dt_by_step)
-            overall_dtable_incremental <- rbind(overall_dtable_incremental, ftype_dt_incremental)
-            overall_summary_dt_by_step <- rbind(overall_summary_dt_by_step, ftype_summary_dt_by_step)
-            overall_summary_dt_incremental <- rbind(overall_summary_dt_incremental, ftype_summary_dt_incremental)
-        }
+                # remove the list of unecessary partitions
+                cl_methods <- names(clustassess_object[[ftype]][[fsize]]$clustering_stability$split_by_k)
+                first_dt_by_k <- TRUE
+                first_dt_by_res <- TRUE
+                # algorithm_names_mapping
 
-        ftype_index <- ftype_index + 1
-    }
+                for (cl_method in cl_methods) {
+                    cl_method_index <- algorithm_names_mapping[[cl_method]]
+                    # --- split by resolution ---
+                    for (res in names(clustassess_object[[ftype]][[fsize]]$clustering_stability$split_by_resolution[[cl_method]])) {
+                        # update the ecc lists for the boxplots
+                        ecc <- clustassess_object[[ftype]][[fsize]]$clustering_stability$split_by_resolution[[cl_method]][[res]]$ecc
+                        ecc_order <- order(ecc)
+                        res_format <- sprintf("%.6f", as.numeric(res))
+                        ecc_list_by_res[[paste(res_format, cl_method, sep = ";")]] <- ecc[ecc_order]
+                        ecc_order_list_by_res[[paste(res_format, cl_method, sep = ";")]] <- Matrix::invPerm(ecc_order)
 
-    feature_ordering$resolution <- stringr::str_sort(resolution_values, numeric = TRUE)
-    # split the data tables by resolution
-    clustassess_object$feature_stability$by_steps <- lapply(feature_ordering$resolution, function(resval) {
-        subdt <- overall_dtable_by_step %>% dplyr::filter(.data$res == resval) # %>% dplyr::arrange(order(.data$ecc))
-        subdt$fsize <- factor(subdt$fsize)
-        subdt$resval <- NULL
+                        # create the summary table for the complex plot
+                        for (n_clusters in names(clustassess_object[[ftype]][[fsize]]$clustering_stability$split_by_resolution[[cl_method]][[res]]$clusters)) {
+                            temp_by_res_summary <- data.frame(
+                                k = as.integer(n_clusters),
+                                res = as.numeric(res),
+                                cl_method = cl_method,
+                                first_freq = clustassess_object[[ftype]][[fsize]]$clustering_stability$split_by_resolution[[cl_method]][[res]]$clusters[[n_clusters]]$partitions[[1]]$freq,
+                                total_freq = sum(
+                                    sapply(
+                                        clustassess_object[[ftype]][[fsize]]$clustering_stability$split_by_resolution[[cl_method]][[res]]$clusters[[n_clusters]]$partitions,
+                                        function(x) {
+                                            x$freq
+                                        }
+                                    )
+                                ),
+                                ecc = summary_function(clustassess_object[[ftype]][[fsize]]$clustering_stability$split_by_resolution[[cl_method]][[res]]$clusters[[n_clusters]]$ecc)
+                            )
 
-        return(subdt)
-    })
-    names(clustassess_object$feature_stability$by_steps) <- feature_ordering$resolution
+                            if (first_dt_by_res) {
+                                first_dt_by_res <- FALSE
+                                by_res_summary <- temp_by_res_summary
+                            } else {
+                                by_res_summary <- rbind(by_res_summary, temp_by_res_summary)
+                            }
+                        }
+                    }
 
-    clustassess_object$feature_stability$incremental <- lapply(feature_ordering$resolution, function(resval) {
-        subdt <- overall_dtable_incremental %>%
-            dplyr::filter(.data$res == resval) %>%
-            dplyr::arrange(order(.data$ecc))
-        subdt$fsize <- factor(subdt$fsize)
+                    # --- split by k ---
+                    structure_list[[cl_method]] <- names(clustassess_object[[ftype]][[fsize]]$clustering_stability$split_by_k[[cl_method]])
+                    for (n_clusters in structure_list[[cl_method]]) {
+                        unique_n_colors <- union(unique_n_colors, n_clusters)
+                        mbs_list[[cl_method]][[n_clusters]] <- as.integer(clustassess_object[[ftype]][[fsize]]$clustering_stability$split_by_k[[cl_method]][[n_clusters]]$partitions[[1]]$mb)
 
-        return(subdt)
-    })
-    names(clustassess_object$feature_stability$incremental) <- feature_ordering$resolution
+                        ecc <- clustassess_object[[ftype]][[fsize]]$clustering_stability$split_by_k[[cl_method]][[n_clusters]]$ecc
+                        ecc_order <- order(ecc)
+                        n_clusters_format <- sprintf("%06d", as.integer(n_clusters))
+                        ecc_list[[paste(n_clusters_format, cl_method, sep = ";")]] <- ecc[ecc_order]
+                        ecc_order_list[[paste(n_clusters_format, cl_method, sep = ";")]] <- Matrix::invPerm(ecc_order)
 
-    clustassess_object$feature_stability$overall <- list(
-        by_step = overall_summary_dt_by_step,
-        incremental = overall_summary_dt_incremental
-    )
-
-    # store the names and ordering of the stable feature sizes
-    for (ftype in names(feature_ordering$original)) {
-        nsteps <- length(clustassess_object[[ftype]]) - 1
-        feature_ordering$stable[[ftype]] <- names(clustassess_object[[ftype]])[seq_len(nsteps)]
-
-        feature_ordering$original_incremental[[ftype]] <- rep(0, length(feature_ordering$original[[ftype]]) - 1)
-
-        for (fsize_index in seq_len(length(feature_ordering$original[[ftype]]) - 1)) {
-            feature_ordering$original_incremental[[ftype]][fsize_index] <- paste(
-                feature_ordering$original[[ftype]][fsize_index],
-                feature_ordering$original[[ftype]][fsize_index + 1],
-                sep = "-"
-            )
-        }
-    }
-
-    # ---
-    # create the clustering data tables
-    # ecc_by_k_dt - columns: clustering method, k and ecc
-    # ecc_by_res_dt - columns: clustering method, res and ecc
-    for (ftype in names(feature_ordering$stable)) {
-        for (fsize in feature_ordering$stable[[ftype]]) {
-            # remove the adjacency matrix, not needed for the shiny app
-            clustassess_object[[ftype]][[fsize]]$adj_matrix <- NULL
-            mbs_list <- list()
-            ecc_list <- list()
-            ecc_list_by_res <- list()
-            ecc_order_list <- list()
-            ecc_order_list_by_res <- list()
-            structure_list <- list()
-
-            # remove the list of unecessary partitions
-            cl_methods <- names(clustassess_object[[ftype]][[fsize]]$clustering_stability$split_by_k)
-            first_dt_by_k <- TRUE
-            first_dt_by_res <- TRUE
-            # algorithm_names_mapping
-
-            for (cl_method in cl_methods) {
-                cl_method_index <- algorithm_names_mapping[[cl_method]]
-                # --- split by resolution ---
-                for (res in names(clustassess_object[[ftype]][[fsize]]$clustering_stability$split_by_resolution[[cl_method]])) {
-                    # update the ecc lists for the boxplots
-                    ecc <- clustassess_object[[ftype]][[fsize]]$clustering_stability$split_by_resolution[[cl_method]][[res]]$ecc
-                    ecc_order <- order(ecc)
-                    res_format <- sprintf("%.6f", as.numeric(res))
-                    ecc_list_by_res[[paste(res_format, cl_method, sep = ";")]] <- ecc[ecc_order]
-                    ecc_order_list_by_res[[paste(res_format, cl_method, sep = ";")]] <- Matrix::invPerm(ecc_order)
-
-                    # create the summary table for the complex plot
-                    for (n_clusters in names(clustassess_object[[ftype]][[fsize]]$clustering_stability$split_by_resolution[[cl_method]][[res]]$clusters)) {
-                        temp_by_res_summary <- data.frame(
+                        temp_by_k_summary <- data.frame(
                             k = as.integer(n_clusters),
-                            res = as.numeric(res),
                             cl_method = cl_method,
-                            first_freq = clustassess_object[[ftype]][[fsize]]$clustering_stability$split_by_resolution[[cl_method]][[res]]$clusters[[n_clusters]]$partitions[[1]]$freq,
+                            n_partitions = length(clustassess_object[[ftype]][[fsize]]$clustering_stability$split_by_k[[cl_method]][[n_clusters]]$partitions),
                             total_freq = sum(
                                 sapply(
-                                    clustassess_object[[ftype]][[fsize]]$clustering_stability$split_by_resolution[[cl_method]][[res]]$clusters[[n_clusters]]$partitions,
+                                    clustassess_object[[ftype]][[fsize]]$clustering_stability$split_by_k[[cl_method]][[n_clusters]]$partitions,
                                     function(x) {
                                         x$freq
                                     }
                                 )
                             ),
-                            ecc = summary_function(clustassess_object[[ftype]][[fsize]]$clustering_stability$split_by_resolution[[cl_method]][[res]]$clusters[[n_clusters]]$ecc)
+                            first_freq = clustassess_object[[ftype]][[fsize]]$clustering_stability$split_by_k[[cl_method]][[n_clusters]]$partitions[[1]]$freq,
+                            ecc = summary_function(ecc)
                         )
 
-                        if (first_dt_by_res) {
-                            first_dt_by_res <- FALSE
-                            by_res_summary <- temp_by_res_summary
+                        if (first_dt_by_k) {
+                            first_dt_by_k <- FALSE
+                            by_k_summary <- temp_by_k_summary
                         } else {
-                            by_res_summary <- rbind(by_res_summary, temp_by_res_summary)
+                            by_k_summary <- rbind(by_k_summary, temp_by_k_summary)
                         }
                     }
                 }
 
-                # --- split by k ---
-                structure_list[[cl_method]] <- names(clustassess_object[[ftype]][[fsize]]$clustering_stability$split_by_k[[cl_method]])
-                for (n_clusters in structure_list[[cl_method]]) {
-                    unique_n_colors <- union(unique_n_colors, n_clusters)
-                    mbs_list[[cl_method]][[n_clusters]] <- as.integer(clustassess_object[[ftype]][[fsize]]$clustering_stability$split_by_k[[cl_method]][[n_clusters]]$partitions[[1]]$mb)
+                clustassess_object[[ftype]][[fsize]]$clustering_stability$split_by_k <- list(
+                    mbs = mbs_list,
+                    summary = by_k_summary %>% dplyr::arrange(.data$k, .data$cl_method),
+                    ecc = ecc_list,
+                    ecc_order = ecc_order_list,
+                    structure_list = structure_list
+                )
 
-                    ecc <- clustassess_object[[ftype]][[fsize]]$clustering_stability$split_by_k[[cl_method]][[n_clusters]]$ecc
-                    ecc_order <- order(ecc)
-                    n_clusters_format <- sprintf("%06d", as.integer(n_clusters))
-                    ecc_list[[paste(n_clusters_format, cl_method, sep = ";")]] <- ecc[ecc_order]
-                    ecc_order_list[[paste(n_clusters_format, cl_method, sep = ";")]] <- Matrix::invPerm(ecc_order)
-
-                    temp_by_k_summary <- data.frame(
-                        k = as.integer(n_clusters),
-                        cl_method = cl_method,
-                        n_partitions = length(clustassess_object[[ftype]][[fsize]]$clustering_stability$split_by_k[[cl_method]][[n_clusters]]$partitions),
-                        total_freq = sum(
-                            sapply(
-                                clustassess_object[[ftype]][[fsize]]$clustering_stability$split_by_k[[cl_method]][[n_clusters]]$partitions,
-                                function(x) {
-                                    x$freq
-                                }
-                            )
-                        ),
-                        first_freq = clustassess_object[[ftype]][[fsize]]$clustering_stability$split_by_k[[cl_method]][[n_clusters]]$partitions[[1]]$freq,
-                        ecc = summary_function(ecc)
-                    )
-
-                    if (first_dt_by_k) {
-                        first_dt_by_k <- FALSE
-                        by_k_summary <- temp_by_k_summary
-                    } else {
-                        by_k_summary <- rbind(by_k_summary, temp_by_k_summary)
-                    }
-                }
+                clustassess_object[[ftype]][[fsize]]$clustering_stability$split_by_resolution <- list(
+                    summary = by_res_summary %>% dplyr::arrange(.data$res, .data$cl_method),
+                    ecc = ecc_list_by_res,
+                    ecc_order = ecc_order_list_by_res
+                )
             }
-
-            clustassess_object[[ftype]][[fsize]]$clustering_stability$split_by_k <- list(
-                mbs = mbs_list,
-                summary = by_k_summary %>% dplyr::arrange(.data$k, .data$cl_method),
-                ecc = ecc_list,
-                ecc_order = ecc_order_list,
-                structure_list = structure_list
-            )
-
-            clustassess_object[[ftype]][[fsize]]$clustering_stability$split_by_resolution <- list(
-                summary = by_res_summary %>% dplyr::arrange(.data$res, .data$cl_method),
-                ecc = ecc_list_by_res,
-                ecc_order = ecc_order_list_by_res
-            )
         }
+        unique_n_colors <- union(unique_n_colors, 1:4)
+        unique_colors <- lapply(unique_n_colors, function(n) {
+            n <- as.integer(n)
+            if (n == 1) {
+                return(single_color)
+            }
+            generate_colours(n, qualpalr_colorspace)
+        })
+        names(unique_colors) <- as.character(unique_n_colors)
+
+        clustassess_object$feature_ordering <- feature_ordering
+        feature_names <- names(feature_ordering$original)
     }
-    unique_n_colors <- union(unique_n_colors, 1:4)
-    unique_colors <- lapply(unique_n_colors, function(n) {
-        n <- as.integer(n)
-        if (n == 1) {
-            return(single_color)
-        }
-        generate_colours(n, qualpalr_colorspace)
-    })
-    names(unique_colors) <- as.character(unique_n_colors)
-
-    clustassess_object$feature_ordering <- feature_ordering
-    feature_names <- names(feature_ordering$original)
 
     # should sparse matrix be used for storing?
     # advantages: lower required space
@@ -380,13 +382,17 @@ write_objects <- function(clustassess_object,
         chunk_matrix <- as.matrix(expression_matrix[start_row:end_row, ])
         gene_vars <- matrixStats::rowVars(chunk_matrix)
         # filter by var threshold
-        chunk_matrix[gene_vars > gene_variance_threshold, ]
+        mask <- gene_vars > gene_variance_threshold
+        if (sum(mask) == 0) {
+            return(NULL)
+        }
+        return(chunk_matrix[gene_vars > gene_variance_threshold, , drop = FALSE])
     })
 
     print("merging chunks")
     expression_matrix <- do.call(rbind, dense_chunks)
     print("chunks merged")
-    
+ 
     gene_avg_expression <- matrixStats::rowMeans2(expression_matrix)
     order_expression <- order(gene_avg_expression, decreasing = TRUE)
     # order the genes based on their average expression
@@ -459,23 +465,25 @@ write_objects <- function(clustassess_object,
         )
     }
 
-    print(glue::glue("[{Sys.time()}] Writing the feature stability"))
+    if (!is.null(clustassess_object)) {
+        print(glue::glue("[{Sys.time()}] Writing the feature stability"))
 
-    # the object for stability
-    rhdf5::h5createFile(stability_file_name)
-    rhdf5::h5write(
-        clustassess_object$feature_stability,
-        stability_file_name,
-        "feature_stability",
-        level = compression_level
-    )
-    rhdf5::h5write(clustassess_object$feature_ordering, stability_file_name, "feature_ordering", level = compression_level)
-    print(glue::glue("[{Sys.time()}] Writing the stability object"))
+        # the object for stability
+        rhdf5::h5createFile(stability_file_name)
+        rhdf5::h5write(
+            clustassess_object$feature_stability,
+            stability_file_name,
+            "feature_stability",
+            level = compression_level
+        )
+        rhdf5::h5write(clustassess_object$feature_ordering, stability_file_name, "feature_ordering", level = compression_level)
+        print(glue::glue("[{Sys.time()}] Writing the stability object"))
 
-    for (feature_type in names(clustassess_object$feature_ordering[[1]])) {
-        rhdf5::h5write(clustassess_object[[feature_type]], stability_file_name, feature_type, level = compression_level)
+        for (feature_type in names(clustassess_object$feature_ordering[[1]])) {
+            rhdf5::h5write(clustassess_object[[feature_type]], stability_file_name, feature_type, level = compression_level)
+        }
+        rhdf5::h5write(unique_colors, stability_file_name, "colors", level = compression_level)
     }
-    rhdf5::h5write(unique_colors, stability_file_name, "colors", level = compression_level)
     print(glue::glue("[{Sys.time()}] Done"))
 
     rhdf5::h5closeAll()
@@ -486,17 +494,23 @@ write_objects <- function(clustassess_object,
 write_shiny_app.Seurat <- function(object,
                                    metadata = NULL,
                                    assay_name,
-                                   clustassess_object,
+                                   clustassess_object = NULL,
                                    project_folder,
                                    compression_level = 6,
                                    summary_function = stats::median,
                                    shiny_app_title = "",
                                    organism_enrichment = "hsapiens",
                                    height_ratio = 0.6,
-                                   qualpalr_colorspace = "pretty") {
+                                   qualpalr_colorspace = "pretty",
+                                   prompt_feature_choice = TRUE) {
+    mtd_df <- object@meta.data
+    if ("umap" %in% names(object@reductions)) {
+        mtd_df <- cbind(mtd_df, object@reductions$umap@cell.embeddings)
+        colnames(mtd_df)[(ncol(mtd_df) - 1):ncol(mtd_df)] <- c("UMAP_1", "UMAP_2")
+    }
     write_shiny_app(
         object = Seurat::GetAssayData(object, assay = assay_name, layer = "data"),
-        metadata = object@meta.data,
+        metadata = mtd_df,
         clustassess_object = clustassess_object,
         project_folder = project_folder,
         compression_level = compression_level,
@@ -504,7 +518,8 @@ write_shiny_app.Seurat <- function(object,
         shiny_app_title = shiny_app_title,
         organism_enrichment = organism_enrichment,
         height_ratio = height_ratio,
-        qualpalr_colorspace = qualpalr_colorspace
+        qualpalr_colorspace = qualpalr_colorspace,
+        prompt_feature_choice = prompt_feature_choice
     )
 }
 
@@ -514,15 +529,15 @@ write_shiny_app.Seurat <- function(object,
 write_shiny_app.default <- function(object,
                                     metadata = NULL,
                                     assay_name = NULL,
-                                    clustassess_object,
+                                    clustassess_object = NULL,
                                     project_folder,
                                     compression_level = 6,
                                     summary_function = stats::median,
                                     shiny_app_title = "",
                                     organism_enrichment = "hsapiens",
                                     height_ratio = 0.6,
-                                    qualpalr_colorspace = "pretty") {
-    # nFeature <- DelayedMatrixStats::colSums2(object > 0)
+                                    qualpalr_colorspace = "pretty",
+                                    prompt_feature_choice = TRUE) {
     if (inherits(object, "dgCMatrix")) {
         nFeature <- Matrix::colSums(object > 0)
     } else {
@@ -532,29 +547,31 @@ write_shiny_app.default <- function(object,
     warning_message <- ""
     shiny_app_title <- paste("ClustAssess ShinyApp -", shiny_app_title)
 
-    for (ftype in names(clustassess_object$feature_stability$by_steps)) {
-        fsizes <- as.integer(names(clustassess_object$feature_stability$by_steps[[ftype]]))
-        fsizes <- fsizes[which(fsizes > stats::median(nFeature))]
-        if (length(fsizes) > 0) {
-            warning_message <- paste0(warning_message, ftype, " - ", paste(fsizes, collapse = ", "), "; ")
+    if (!is.null(clustassess_object)) {
+        for (ftype in names(clustassess_object$feature_stability$by_steps)) {
+            fsizes <- as.integer(names(clustassess_object$feature_stability$by_steps[[ftype]]))
+            fsizes <- fsizes[which(fsizes > stats::median(nFeature))]
+            if (length(fsizes) > 0) {
+                warning_message <- paste0(warning_message, ftype, " - ", paste(fsizes, collapse = ", "), "; ")
+            }
         }
-    }
 
-    if (stringr::str_length(warning_message) > 0) {
-        while (TRUE) {
-            warning(glue::glue("WARNING: The following configurations -- {warning_message} have a size above the average number of nFeatures per cell - {median(nFeature)}.\nIncreasing the number of features above the average will lead to introduction of noise in the data, thus we recommed re-running ClustAssess with lower values for the feature steps.\nPlease type `yes` or `no` if you want to continue creating the ClustAssess shiny app."), immediate. = TRUE)
-            if (interactive()) {
-                user_input <- readline()
-            } else {
-                user_input <- readLines("stdin", n = 1)
-            }
+        if (prompt_feature_choice && stringr::str_length(warning_message) > 0) {
+            while (TRUE) {
+                warning(glue::glue("WARNING: The following configurations -- {warning_message} have a size above the average number of nFeatures per cell - {median(nFeature)}.\nIncreasing the number of features above the average will lead to introduction of noise in the data, thus we recommed re-running ClustAssess with lower values for the feature steps.\nPlease type `yes` or `no` if you want to continue creating the ClustAssess shiny app."), immediate. = TRUE)
+                if (interactive()) {
+                    user_input <- readline()
+                } else {
+                    user_input <- readLines("stdin", n = 1)
+                }
 
-            if (user_input == "no") {
-                return()
-            }
+                if (user_input == "no") {
+                    return()
+                }
 
-            if (user_input == "yes") {
-                break
+                if (user_input == "yes") {
+                    break
+                }
             }
         }
     }
@@ -704,8 +721,10 @@ write_shiny_app.default <- function(object,
             if (tab_number == 1) {
             server_landing_page(\"landing_page\", height_ratio, reactive(input$dimension), session, \"", organism_enrichment, "\")
             }
-
-            if (tab_number == 2) {
+            
+            ",
+            ifelse(is.null(clustassess_object), "",
+            "if (tab_number == 2) {
             fchoice(server_dimensionality_reduction(\"dim_reduc\", session))
             }
 
@@ -715,8 +734,8 @@ write_shiny_app.default <- function(object,
 
             if (tab_number == 4) {
             cchoice(server_graph_clustering(\"graph_clust\", fchoice(), session))
-            }
-
+            }"),
+            "
             if (tab_number == 5) {
             server_comparisons(\"comparison\", fchoice(), cchoice())
             }
