@@ -315,6 +315,22 @@ compar_annotation_info <- function(session) {
     )
 }
 
+compar_split_info <- function(session) {
+    shiny::showModal(
+        shiny::modalDialog(
+            shiny::h3(shiny::strong("Metadata splitting")),
+            shiny::br(),
+            shiny::h5("Here the user can create new metadata by combining the unique values from two discrete metadata columns."),
+            shiny::h5("\n"),
+            shiny::h5("This can be done by selecting any of the existing metadata columns. The two metadata columns must be different. By default, the name of the new column will be the concatenation of the two selected columns, but this behaviour can be changed by the user. The new column name shouldn't overlap with the name of an existing one."),
+            shiny::h5("\n"),
+            shiny::h5("Example: column A has values a, b, c; column B has values 1, 2, 3. After splitting, the new column A_B will have values a_1, b_2, c_3."),
+            easyClose = TRUE
+        ),
+        session
+    )
+}
+
 compar_distribution_info <- function(session) {
     shiny::showModal(
         shiny::modalDialog(
@@ -331,16 +347,16 @@ compar_distribution_info <- function(session) {
     )
 }
 
-compar_metadata_jsi_info <- function(session) {
+compar_metadata_comparison_info <- function(session) {
     shiny::showModal(
         shiny::modalDialog(
-            shiny::h3(shiny::strong("Metadata JSI")),
+            shiny::h3(shiny::strong("Metadata comparison - Contingency heatmap and Proportion barplot")),
             shiny::br(),
-            shiny::h5("Here the user can visualise the relationship between the discrete metadata and the clustering results."),
+            shiny::h5("Here the user can visualise the relationship between two discrete metadatas"),
             shiny::h5("\n"),
-            shiny::h5("This can be useful in the context of identifying relationships between 'super-clusters' and more fine-grained metadata."),
+            shiny::h5("This plot aims to showcase the behaviour of the individual clusters on the different partitions. JSI is calculated for the cell barcodes for every cluster, in both configurations, in a pair-wise manner."),
             shiny::h5("\n"),
-            shiny::h5("The displayed information can be either the number of common cells, or the Jaccard Similarity Index (JSI); the JSI is calculated row-wise."),
+            shiny::h5("The proportion can be visualised, both as number of cells as percentages, in a barplot."),
             easyClose = TRUE
         ),
         session
